@@ -36,7 +36,7 @@
 
 /* This is here so we can quickly disable windowless support. For now it's 
  * missing some features (cursor support) and redraws seem to be buggy */
-#define ENABLE_WINDOWLESS
+//#define DISABLE_WINDOWLESS
 
 NPNetscapeFuncs mozilla_funcs;
 
@@ -127,7 +127,7 @@ NP_GetValue (void* reserved, NPPVariable var, void* out)
       break;
     case NPPVpluginDescriptionString:
       /* FIXME: find a way to encode the Swfdec version without breaking stupid JS scripts */
-      *val = (char *) "Shockwave Flash 9.0 r100";
+      *val = (char *) "Shockwave Flash 9.0 r999";
       break;
     case NPPVpluginNeedsXEmbed:
       *((PRBool*) val) = PR_FALSE;
@@ -179,7 +179,7 @@ swfdec_mozilla_make_sure_this_thing_stays_in_memory (void)
   return TRUE;
 }
 
-#ifdef ENABLE_WINDOWLESS
+#ifndef DISABLE_WINDOWLESS
 /* returns true if this instance can run windowless */
 static gboolean
 plugin_try_windowless (NPP instance)
@@ -230,7 +230,7 @@ plugin_new (NPMIMEType mime_type, NPP instance,
    * won't be unloaded, i.e. NPPVpluginKeepLibraryInMemory was successful */
   swfdec_init ();
 
-#ifdef ENABLE_WINDOWLESS
+#ifndef DISABLE_WINDOWLESS
   /* parse pre-creation properties */
   for (i = 0; i < argc; i++) {
     if (g_ascii_strcasecmp (argn[i], "wmode") == 0) {
@@ -340,7 +340,7 @@ plugin_destroy_stream_cb (NPP instance, NPStream* stream, NPReason reason)
     return NPERR_INVALID_INSTANCE_ERROR;
 
   swfmoz_loader_ensure_open (stream->pdata);
-  swfdec_stream_eof (stream->pdata);
+  swfdec_stream_close (stream->pdata);
   SWFMOZ_LOADER (stream->pdata)->stream = NULL;
   g_object_unref (stream->pdata);
   return NPERR_NO_ERROR;
@@ -396,7 +396,8 @@ plugin_set_window (NPP instance, NPWindow *window)
 
   if (window) {
     plugin_x11_setup_windowed (instance->pdata, (Window) window->window, 
-	window->x, window->y, window->width, window->height);
+	window->x, window->y, window->width, window->height,
+	((NPSetWindowCallbackStruct *)window->ws_info)->visual);
   } else {
     plugin_x11_teardown (instance->pdata);
   }
