@@ -64,6 +64,31 @@ swfmoz_player_menu_notify_audio (SwfdecGtkPlayer *player, GParamSpec *pspec,
     gtk_check_menu_item_set_active (item, swfdec_gtk_player_get_audio_enabled (player));
 }
 
+static void
+swfmoz_player_menu_autoplay_always_toggled (GtkCheckMenuItem *item,
+    SwfmozPlayer* player)
+{
+  gboolean item_active = gtk_check_menu_item_get_active (item);
+  swfmoz_config_set_global_autoplay (player->config, item_active);
+}
+
+static void
+swfmoz_player_menu_autoplay_remember_last_toggled (GtkCheckMenuItem *item,
+    SwfmozPlayer* player)
+{
+  gboolean item_active = gtk_check_menu_item_get_active (item);
+  if (item_active && swfmoz_config_has_global_key (player->config))
+    swfmoz_config_remove_global_autoplay (player->config);
+}
+
+static void
+swfmoz_player_menu_autoplay_never_toggled (GtkCheckMenuItem *item,
+    SwfmozPlayer* player)
+{
+  gboolean item_active = gtk_check_menu_item_get_active (item);
+  swfmoz_config_set_global_autoplay (player->config, !item_active);
+}
+
 static void
 swfmoz_player_menu_about (GtkMenuItem *item, SwfmozPlayer *player)
 {
@@ -81,7 +106,7 @@ swfmoz_player_menu_about (GtkMenuItem *item, SwfmozPlayer *player)
   };
 
   gtk_show_about_dialog (NULL, "program-name", "Swfdec",
-      "logo-icon-name", "swfdec",
+      "logo-icon-name", "swfdec-mozilla",
       "authors", authors,
       "artists", artists,
       "comments", "Play Flash content in your browser",
@@ -95,31 +120,62 @@ static void
 swfmoz_player_popup_menu (SwfmozPlayer *player)
 {
   if (player->menu == NULL) {
-    GtkWidget *item;
+    GtkWidget *item, *submenu;
 
     player->menu = GTK_MENU (gtk_menu_new ());
     g_object_ref_sink (player->menu);
     
     item = gtk_check_menu_item_new_with_mnemonic ("Playing");
-    g_signal_connect (item, "toggled", 
+    g_signal_connect (item, "toggled",
 	G_CALLBACK (swfmoz_player_menu_playing_toggled), player);
     g_signal_connect (player, "notify::playing",
 	G_CALLBACK (swfmoz_player_menu_notify_playing), item);
-    gtk_check_menu_item_set_active (GTK_CHECK_MENU_ITEM (item), 
+    gtk_check_menu_item_set_active (GTK_CHECK_MENU_ITEM (item),
 	swfdec_gtk_player_get_playing (SWFDEC_GTK_PLAYER (player)));
     gtk_widget_show (item);
     gtk_menu_shell_append (GTK_MENU_SHELL (player->menu), item);
 
     item = gtk_check_menu_item_new_with_mnemonic ("Enable Audio");
-    g_signal_connect (item, "toggled", 
+    g_signal_connect (item, "toggled",
 	G_CALLBACK (swfmoz_player_menu_audio_toggled), player);
     g_signal_connect (player, "notify::audio-enabled",
 	G_CALLBACK (swfmoz_player_menu_notify_audio), item);
-    gtk_check_menu_item_set_active (GTK_CHECK_MENU_ITEM (item), 
+    gtk_check_menu_item_set_active (GTK_CHECK_MENU_ITEM (item),
 	swfdec_gtk_player_get_audio_enabled (SWFDEC_GTK_PLAYER (player)));
     gtk_widget_show (item);
     gtk_menu_shell_append (GTK_MENU_SHELL (player->menu), item);
 
+    submenu = gtk_menu_new();
+    item = gtk_radio_menu_item_new_with_mnemonic (NULL, "Always");
+    g_signal_connect (item, "toggled",
+        G_CALLBACK (swfmoz_player_menu_autoplay_always_toggled), player);
+    gtk_check_menu_item_set_active (GTK_CHECK_MENU_ITEM (item),
+	swfmoz_config_read_autoplay (player->config, "global", FALSE));
+    gtk_widget_show (item);
+    gtk_menu_shell_append (GTK_MENU_SHELL (submenu), item);
+
+    item = gtk_radio_menu_item_new_with_mnemonic_from_widget (GTK_RADIO_MENU_ITEM (item),
+        "Remember last choice");
+    g_signal_connect (item, "toggled",
+	G_CALLBACK (swfmoz_player_menu_autoplay_remember_last_toggled), player);
+    gtk_check_menu_item_set_active (GTK_CHECK_MENU_ITEM (item),
+        !swfmoz_config_has_global_key (player->config));
+    gtk_widget_show (item);
+    gtk_menu_shell_append (GTK_MENU_SHELL (submenu), item);
+
+    item = gtk_radio_menu_item_new_with_mnemonic_from_widget (GTK_RADIO_MENU_ITEM (item), "Never");
+    g_signal_connect (item, "toggled",
+        G_CALLBACK (swfmoz_player_menu_autoplay_never_toggled), player);
+    gtk_check_menu_item_set_active (GTK_CHECK_MENU_ITEM (item),
+	!swfmoz_config_read_autoplay (player->config, "global", TRUE));
+    gtk_widget_show (item);
+    gtk_menu_shell_append (GTK_MENU_SHELL (submenu), item);
+
+    item = gtk_menu_item_new_with_label ("Autoplay");
+    gtk_widget_show (item);
+    gtk_menu_item_set_submenu (GTK_MENU_ITEM (item), submenu);
+    gtk_menu_shell_append (GTK_MENU_SHELL (player->menu), item);
+
     item = gtk_separator_menu_item_new ();
     gtk_widget_show (item);
     gtk_menu_shell_append (GTK_MENU_SHELL (player->menu), item);
@@ -182,12 +238,15 @@ swfmoz_player_idle_redraw (gpointer playerp)
 }
 
 static void
-swfmoz_player_redraw (SwfmozPlayer *player, const SwfdecRectangle *extents, 
+swfmoz_player_redraw (SwfmozPlayer *player,
     const SwfdecRectangle *rects, guint n_rects, gpointer unused)
 {
   GdkRegion *region;
   guint i;
 
+  if (player->target == NULL)
+    return;
+
   if (player->repaint)
     region = player->repaint;
   else
@@ -211,7 +270,7 @@ swfmoz_player_redraw (SwfmozPlayer *player, const SwfdecRectangle *extents,
 
 static SwfdecBuffer *
 swfmoz_player_format_headers (guint header_count, const char **header_names,
-    const char **header_values)
+    const char **header_values, gsize content_length)
 {
   GString *string;
   guint i;
@@ -220,14 +279,16 @@ swfmoz_player_format_headers (guint header_count, const char **header_names,
   g_return_val_if_fail (header_count == 0 || header_names != NULL, NULL);
   g_return_val_if_fail (header_count == 0 || header_values != NULL, NULL);
 
-  string = g_string_new ("");
+  string = g_string_new ("Content-Length: ");
+  g_string_append_printf (string, "%"G_GSIZE_FORMAT, content_length);
+  g_string_append (string, "\n");
   for (i = 0; i < header_count; i++) {
     g_string_append (string, header_names[i]);
     g_string_append (string, ": ");
     g_string_append (string, header_values[i]);
     g_string_append (string, "\n");
   }
-  g_string_append (string, "\n\n");
+  g_string_append (string, "\n");
 
   len = string->len;
   return swfdec_buffer_new_for_data (
@@ -247,7 +308,7 @@ swfmoz_player_add_headers (SwfdecBuffer *data, guint header_count,
 
   queue = swfdec_buffer_queue_new ();
   swfdec_buffer_queue_push (queue, swfmoz_player_format_headers (
-	header_count, header_names, header_values));
+	header_count, header_names, header_values, data->length));
   swfdec_buffer_queue_push (queue, swfdec_buffer_ref (data));
 
   buffer = swfdec_buffer_queue_pull (queue,
@@ -277,7 +338,7 @@ static void
 swfmoz_player_invalidate (SwfmozPlayer *player)
 {
   SwfdecRectangle rect = { 0, 0, player->target_rect.width, player->target_rect.height };
-  swfmoz_player_redraw (player, &rect, &rect, 1, NULL);
+  swfmoz_player_redraw (player, &rect, 1, NULL);
 }
 
 /* function stolen from SwfdecGtkWidget */
@@ -397,6 +458,33 @@ swfmoz_player_notify (SwfmozPlayer *player, GParamSpec *pspec, gpointer unused)
   }
 }
 
+static gboolean
+swfmoz_player_query_size (SwfmozPlayer *player, gboolean fullscreen, 
+    int *width, int *height)
+{
+  if (fullscreen) {
+    GdkWindow *window = plugin_get_window (player->instance);
+    GdkScreen *screen;
+    GdkRectangle rect;
+    int monitor;
+
+    if (window == NULL)
+      return FALSE;
+    screen = gdk_drawable_get_screen (GDK_DRAWABLE (window));
+    monitor = gdk_screen_get_monitor_at_window (screen, window);
+    gdk_screen_get_monitor_geometry (screen, monitor, &rect);
+    *width = rect.width;
+    *height = rect.height;
+  } else if (player->target == NULL) {
+    *width = -1;
+    *height = -1;
+  } else {
+    *width = player->target_rect.width;
+    *height = player->target_rect.height;
+  }
+  return TRUE;
+}
+
 static void
 swfmoz_player_get_property (GObject *object, guint param_id, GValue *value, 
     GParamSpec * pspec)
@@ -423,6 +511,18 @@ swfmoz_player_set_property (GObject *object, guint param_id, const GValue *value
   }
 }
 
+static void
+swfmoz_player_clear_repaints (SwfmozPlayer *player)
+{
+  if (player->repaint_source) {
+    g_source_destroy (player->repaint_source);
+    g_source_unref (player->repaint_source);
+    player->repaint_source = NULL;
+    gdk_region_destroy (player->repaint);
+    player->repaint = NULL;
+  }
+}
+
 static void
 swfmoz_player_dispose (GObject *object)
 {
@@ -446,17 +546,12 @@ swfmoz_player_dispose (GObject *object)
   g_signal_handlers_disconnect_by_func (player, swfmoz_player_redraw, NULL);
   g_signal_handlers_disconnect_by_func (player, swfmoz_player_launch, NULL);
   g_signal_handlers_disconnect_by_func (player, swfmoz_player_notify, NULL);
+  g_signal_handlers_disconnect_by_func (player, swfmoz_player_query_size, NULL);
   if (player->target) {
     g_object_unref (player->target);
     player->target = NULL;
   }
-  if (player->repaint_source) {
-    g_source_destroy (player->repaint_source);
-    g_source_unref (player->repaint_source);
-    player->repaint_source = NULL;
-    gdk_region_destroy (player->repaint);
-    player->repaint = NULL;
-  }
+  swfmoz_player_clear_repaints (player);
   if (player->initial) {
     g_object_unref (player->initial);
     player->initial = NULL;
@@ -488,11 +583,11 @@ swfmoz_player_init (SwfmozPlayer *player)
   g_signal_connect (player, "invalidate", G_CALLBACK (swfmoz_player_redraw), NULL);
   g_signal_connect (player, "launch", G_CALLBACK (swfmoz_player_launch), NULL);
   g_signal_connect (player, "notify", G_CALLBACK (swfmoz_player_notify), NULL);
+  g_signal_connect (player, "query-size", G_CALLBACK (swfmoz_player_query_size), NULL);
   player->context = g_main_context_default ();
 
   player->loaders = GTK_TREE_MODEL (gtk_list_store_new (SWFMOZ_LOADER_N_COLUMNS,
-      SWFMOZ_TYPE_LOADER, G_TYPE_STRING, G_TYPE_STRING,
-      G_TYPE_BOOLEAN, G_TYPE_BOOLEAN, G_TYPE_STRING));
+      SWFMOZ_TYPE_LOADER, G_TYPE_STRING, G_TYPE_STRING, G_TYPE_STRING));
 }
 
 SwfdecPlayer *
@@ -519,17 +614,18 @@ static void
 swfmoz_player_loaders_update (GtkListStore *store, GtkTreeIter *iter, SwfdecLoader *loader)
 {
   goffset loaded, size;
-  gboolean eof, error;
+  gboolean error;
   const SwfdecURL *url;
+  SwfdecPlayer *player;
   const char *url_string;
   char *str_loaded, *str_size;
-  gchar *status;
+  gchar *status, *s = NULL;
 
   loaded = swfdec_loader_get_loaded (loader);
   size = swfdec_loader_get_size (loader);
 
   /* FIXME: swfdec needs a function for this */
-  g_object_get (G_OBJECT (loader), "eof", &eof, "error", &error, NULL);
+  g_object_get (G_OBJECT (loader), "error", &error, NULL);
 
   if (error == TRUE) {
     status = g_strdup("error");
@@ -537,19 +633,29 @@ swfmoz_player_loaders_update (GtkListStore *store, GtkTreeIter *iter, SwfdecLoad
     str_loaded = g_format_size_for_display(loaded);
     str_size = g_format_size_for_display(size);
 
-    if (size == loaded)
+    if (swfdec_stream_is_complete (SWFDEC_STREAM (loader))) {
       status = g_strdup_printf("%s", str_loaded);
-    else if (size < 0)
+    } else if (size < 0 || (size < loaded)) {
       status = g_strdup_printf("at %s", str_loaded);
-    else
+    } else {
       status = g_strdup_printf("%s of %s", str_loaded, str_size);
+    }
 
     g_free (str_loaded);
     g_free (str_size);
   }
 
   url = swfdec_loader_get_url (loader);
-  if (url) {
+  player = (*SWFMOZ_LOADER (loader)->instance)->pdata;
+  if (url && SWFMOZ_LOADER (loader)->initial && swfdec_player_get_variables (player)) {
+    /* This auto-appends the FlashVars to the reported URL. You should be able
+     * to copy/paste that URL easily without breakage that way 
+     * (minus cookies and referer) */
+    s = g_strconcat (swfdec_url_get_url (url),
+	swfdec_url_get_query (url) ? "&" : "?",
+	swfdec_player_get_variables (player), NULL);
+    url_string = s;
+  } else if (url) {
     url_string = swfdec_url_get_url (url);
   } else if (SWFMOZ_LOADER (loader)->stream) {
     url_string = SWFMOZ_LOADER (loader)->stream->url;
@@ -561,13 +667,12 @@ swfmoz_player_loaders_update (GtkListStore *store, GtkTreeIter *iter, SwfdecLoad
   gtk_list_store_set (store, iter,
     SWFMOZ_LOADER_COLUMN_LOADER, loader,
     SWFMOZ_LOADER_COLUMN_URL, url_string,
-    SWFMOZ_LOADER_COLUMN_EOF, eof,
-    SWFMOZ_LOADER_COLUMN_ERROR, error,
     SWFMOZ_LOADER_COLUMN_TYPE, swfmoz_loader_get_data_type_string (loader),
     SWFMOZ_LOADER_COLUMN_STATUS, status,
     -1);
 
-  g_free(status);
+  g_free (status);
+  g_free (s);
 }
 
 static gboolean
@@ -643,7 +748,7 @@ swfmoz_player_add_loader (SwfmozPlayer *player, SwfmozLoader *loader)
 
 void
 swfmoz_player_set_target (SwfmozPlayer *player, GdkWindow *target, 
-    int x, int y, int width, int height)
+    int x, int y, int width, int height, Visual *visual)
 {
   g_return_if_fail (SWFMOZ_IS_PLAYER (player));
   g_return_if_fail (target == NULL || GDK_IS_WINDOW (target));
@@ -653,6 +758,7 @@ swfmoz_player_set_target (SwfmozPlayer *player, GdkWindow *target,
       g_object_unref (player->target);
     }
     player->target = target;
+    swfmoz_player_clear_repaints (player);
     if (target) {
       cairo_t *cr;
       SwfdecRenderer *renderer;
@@ -678,6 +784,7 @@ swfmoz_player_set_target (SwfmozPlayer *player, GdkWindow *target,
   player->target_rect.y = y;
   player->target_rect.width = width;
   player->target_rect.height = height;
+  player->target_visual = visual;
   swfdec_player_set_size (SWFDEC_PLAYER (player), width, height);
 }
 
@@ -793,8 +900,7 @@ swfmoz_player_render (SwfmozPlayer *player, cairo_t *cr, GdkRegion *region)
   }
   cairo_translate (cr, player->target_rect.x, player->target_rect.y);
   gdk_region_get_clipbox (region, &rect);
-  swfdec_player_render (SWFDEC_PLAYER (player), cr, rect.x - player->target_rect.x,
-      rect.y - player->target_rect.y, rect.width, rect.height);
+  swfdec_player_render (SWFDEC_PLAYER (player), cr);
   /* paint optional pause sign */
   if (!swfdec_gtk_player_get_playing (SWFDEC_GTK_PLAYER (player)) &&
       swfdec_player_get_url (SWFDEC_PLAYER (player)) != NULL) {
@@ -924,6 +1030,7 @@ swfmoz_player_remove (SwfmozPlayer *player)
   swfdec_gtk_player_set_playing (SWFDEC_GTK_PLAYER (player), FALSE);
   swfdec_gtk_player_set_audio_enabled (SWFDEC_GTK_PLAYER (player), FALSE);
   swfmoz_dialog_remove (player);
+  player->instance = NULL;
   g_object_unref (player);
 }
 
